@@ -38,13 +38,13 @@
 #define NOTIFY_SUBSCRIPTION_TIME 20000
 
 /* Max length of the body part of a message we show in the
- * notification. 
+ * notification.
  */
-#define NOTIFY_MESSAGE_MAX_LEN 64 
+#define NOTIFY_MESSAGE_MAX_LEN 64
 
 /* Time to wait before we use notification for an account after it has
  * gone online/offline, so we don't spam the sound with online's, etc
- */ 
+ */
 #define NOTIFY_WAIT_TIME 10000
 
 static const gchar *       notify_get_status_from_presence        (GossipPresence     *presence);
@@ -70,6 +70,7 @@ static void                notify_protocol_connected_cb           (GossipSession
 static void                notify_protocol_disconnected_cb        (GossipSession      *session,
 								   GossipAccount      *account,
 								   GossipProtocol     *protocol,
+								   gint                reason,
 								   gpointer            user_data);
 static void                notify_contact_presence_updated_cb     (GossipSession      *session,
 								   GossipContact      *contact,
@@ -90,12 +91,13 @@ enum {
 	NOTIFY_SHOW_ROSTER
 };
 
-static gboolean    inited = FALSE;
-static GHashTable *account_states = NULL;
-static GHashTable *contact_states = NULL;
-static GHashTable *message_notifications = NULL;
-static GHashTable *event_notifications = NULL;
-static GtkWidget  *attach_widget = NULL;
+static gboolean       inited = FALSE;
+static GHashTable    *account_states = NULL;
+static GHashTable    *contact_states = NULL;
+static GHashTable    *message_notifications = NULL;
+static GHashTable    *event_notifications = NULL;
+static GtkWidget     *attach_widget = NULL;
+static GtkStatusIcon *attach_status_icon = NULL;
 
 static const gchar *
 notify_get_status_from_presence (GossipPresence *presence)
@@ -109,7 +111,7 @@ notify_get_status_from_presence (GossipPresence *presence)
 		state = gossip_presence_get_state (presence);
 		status = gossip_presence_state_get_default_status (state);
 	}
-	
+
 	return status;
 }
 
@@ -142,11 +144,11 @@ notify_get_is_busy (void)
 {
 	GossipPresence      *presence;
 	GossipPresenceState  state;
-	
-	presence = gossip_session_get_presence (gossip_app_get_session ());
-        state = gossip_presence_get_state (presence);
 
-        if (state == GOSSIP_PRESENCE_STATE_BUSY) {
+	presence = gossip_session_get_presence (gossip_app_get_session ());
+	state = gossip_presence_get_state (presence);
+
+	if (state == GOSSIP_PRESENCE_STATE_BUSY) {
 		return TRUE;
 	}
 
@@ -178,8 +180,8 @@ notify_contact_online (GossipContact *contact)
 	if (notify_get_is_busy ()) {
 		return;
 	}
-	
-	gossip_debug (DEBUG_DOMAIN, "Setting up notification for online contact:'%s'", 
+
+	gossip_debug (DEBUG_DOMAIN, "Setting up notification for online contact:'%s'",
 		      gossip_contact_get_id (contact));
 
 	presence = gossip_contact_get_active_presence (contact);
@@ -195,10 +197,10 @@ notify_contact_online (GossipContact *contact)
 
 	if (!pixbuf) {
 		/* Fall back to presence state */
-		pixbuf = gossip_pixbuf_for_presence (presence); 
+		pixbuf = gossip_pixbuf_for_presence (presence);
 	}
 
-	title = g_markup_printf_escaped (_("%s has come online"), 
+	title = g_markup_printf_escaped (_("%s has come online"),
 					 gossip_contact_get_name (contact));
 	status = notify_get_status_from_presence (presence);
 
@@ -211,8 +213,8 @@ notify_contact_online (GossipContact *contact)
 			  G_CALLBACK (notify_closed_cb),
 			  NULL);
 
-	if (attach_widget) {
-		notify_notification_attach_to_widget (notify, attach_widget);
+	if (attach_status_icon) {
+		notify_notification_attach_to_status_icon (notify, attach_status_icon);
 	}
 
 	notify_notification_add_action (notify, "default", _("Default"),
@@ -270,14 +272,14 @@ notify_subscription_request_show (GossipContact *contact)
 
 	if (!pixbuf) {
 		/* Fall back to presence state */
-		pixbuf = gossip_pixbuf_from_stock (GTK_STOCK_DIALOG_QUESTION, 
+		pixbuf = gossip_pixbuf_from_stock (GTK_STOCK_DIALOG_QUESTION,
 						   GTK_ICON_SIZE_MENU);
 	}
 
 	notify = notify_notification_new (_("Subscription request"),
 					  message,
 					  NULL,
-					  attach_widget);
+					  NULL);
 	g_free (message);
 
 	notify_notification_set_urgency (notify, NOTIFY_URGENCY_NORMAL);
@@ -319,7 +321,7 @@ notify_new_message_default_cb (NotifyNotification *notify,
 	if (event) {
 		message = GOSSIP_MESSAGE (gossip_event_get_data (event));
 		contact = gossip_message_get_sender (message);
-	
+
 		gossip_event_manager_activate (event_manager, event);
 
 		g_hash_table_remove (event_notifications, notify);
@@ -346,8 +348,12 @@ notify_new_message_contact_cb (NotifyNotification *notify,
 
 		message = GOSSIP_MESSAGE (gossip_event_get_data (event));
 		contact = gossip_message_get_sender (message);
-
+		
 		parent = GTK_WINDOW (gossip_app_get_window ());
+		if (!gossip_window_get_is_visible (parent)) {
+			parent = NULL;
+		}
+
 		gossip_contact_info_dialog_show (contact, parent);
 
 		g_hash_table_remove (event_notifications, notify);
@@ -362,7 +368,7 @@ notify_new_message_contact_cb (NotifyNotification *notify,
 static NotifyNotification *
 notify_new_message (GossipEventManager *event_manager,
 		    GossipMessage      *message)
-{ 
+{
 	GossipContact      *contact;
 	NotifyNotification *notify;
 	GdkPixbuf          *pixbuf = NULL;
@@ -378,16 +384,16 @@ notify_new_message (GossipEventManager *event_manager,
 
 	body_copy = g_strdup (gossip_message_get_body (message));
 	body_stripped = g_strstrip (body_copy);
-	
+
 	len = g_utf8_strlen (body_stripped, -1);
 	if (len < 1) {
-		gossip_debug (DEBUG_DOMAIN, "Ignoring new message from:'%s', no message content", 
+		gossip_debug (DEBUG_DOMAIN, "Ignoring new message from:'%s', no message content",
 			      gossip_contact_get_id (contact));
 		g_free (body_copy);
 		return NULL;
 	}
 
-	gossip_debug (DEBUG_DOMAIN, "Setting up notification for new message from:'%s'", 
+	gossip_debug (DEBUG_DOMAIN, "Setting up notification for new message from:'%s'",
 		      gossip_contact_get_id (contact));
 
 	show_avatars = FALSE;
@@ -404,7 +410,7 @@ notify_new_message (GossipEventManager *event_manager,
 		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE, GTK_ICON_SIZE_MENU);
 	}
 
-	title = g_strdup_printf (_("New message from %s"), 
+	title = g_strdup_printf (_("New message from %s"),
 				 gossip_contact_get_name (contact));
 
 	if (len >= NOTIFY_MESSAGE_MAX_LEN) {
@@ -412,7 +418,7 @@ notify_new_message (GossipEventManager *event_manager,
 
 		end = g_utf8_offset_to_pointer (body_stripped, NOTIFY_MESSAGE_MAX_LEN);
 		end[0] = '\0';
-		
+
 		str = g_markup_printf_escaped ("%s...", body_stripped);
 	} else {
 		str = g_markup_printf_escaped ("%s", body_stripped);
@@ -428,10 +434,10 @@ notify_new_message (GossipEventManager *event_manager,
 			  G_CALLBACK (notify_closed_cb),
 			  NULL);
 
-	if (attach_widget) {
-		notify_notification_attach_to_widget (notify, attach_widget);
+	if (attach_status_icon) {
+		notify_notification_attach_to_status_icon (notify, attach_status_icon);
 	}
-	
+
 	notify_notification_add_action (notify, "default", _("Default"),
 					(NotifyActionCallback) notify_new_message_default_cb,
 					g_object_ref (event_manager), NULL);
@@ -445,7 +451,7 @@ notify_new_message (GossipEventManager *event_manager,
 			(NotifyActionCallback) notify_new_message_contact_cb,
 			g_object_ref (event_manager), NULL);
 	}
-	
+
 	if (!notify_notification_show (notify, &error)) {
 		g_warning ("Failed to send notification: %s",
 			   error->message);
@@ -479,7 +485,7 @@ notify_disconnected_message_foreach (GossipContact  *contact,
 	GossipAccount *contact_account;
 
 	contact_account = gossip_contact_get_account (contact);
-	
+
 	if (gossip_account_equal (contact_account, account)) {
 		return TRUE;
 	}
@@ -495,7 +501,7 @@ notify_disconnected_contact_foreach (GossipContact  *contact,
 	GossipAccount *contact_account;
 
 	contact_account = gossip_contact_get_account (contact);
-	
+
 	if (gossip_account_equal (contact_account, account)) {
 		return TRUE;
 	}
@@ -507,14 +513,15 @@ static void
 notify_protocol_disconnected_cb (GossipSession  *session,
 				 GossipAccount  *account,
 				 GossipProtocol *protocol,
+				 gint            reason,
 				 gpointer        user_data)
 {
 	g_hash_table_remove (account_states, account);
-	
+
 	g_hash_table_foreach_remove (message_notifications,
 				     (GHRFunc) notify_disconnected_message_foreach,
 				     account);
-	
+
 	g_hash_table_foreach_remove (contact_states,
 				     (GHRFunc) notify_disconnected_contact_foreach,
 				     account);
@@ -526,17 +533,19 @@ notify_protocol_connected_cb (GossipSession  *session,
 			      GossipProtocol *protocol,
 			      gpointer        user_data)
 {
-	guint id;
+	guint        id;
+	const gchar *account_id;
 
 	if (g_hash_table_lookup (account_states, account)) {
 		return;
 	}
 
+	gossip_account_param_get (account, "id", &account_id, NULL);
 	gossip_debug (DEBUG_DOMAIN, "Account update, account:'%s' is now online",
-		      gossip_account_get_id (account));
+		      account_id);
 
 	id = g_timeout_add (NOTIFY_WAIT_TIME,
-			    (GSourceFunc) notify_protocol_timeout_cb, 
+			    (GSourceFunc) notify_protocol_timeout_cb,
 			    account);
 	g_hash_table_insert (account_states, account, GUINT_TO_POINTER (id));
 }
@@ -558,26 +567,26 @@ notify_contact_presence_updated_cb (GossipSession *session,
 			gossip_debug (DEBUG_DOMAIN, "Presence update, contact:'%s' is now offline",
 				      gossip_contact_get_id (contact));
 		}
-			
+
 		g_hash_table_remove (contact_states, contact);
 	} else {
 		GossipAccount *account;
-		
+
 		account = gossip_contact_get_account (contact);
 
 		/* Only show notifications after being online for some
 		 * time instead of spamming notifications each time we
 		 * connect.
 		 */
-		if (!g_hash_table_lookup (account_states, account) && 
+		if (!g_hash_table_lookup (account_states, account) &&
 		    !g_hash_table_lookup (contact_states, contact)) {
 			gossip_debug (DEBUG_DOMAIN, "Presence update, contact:'%s' is now online",
 				      gossip_contact_get_id (contact));
 			notify_contact_online (contact);
 		}
-		
-		g_hash_table_insert (contact_states, 
-				     g_object_ref (contact), 
+
+		g_hash_table_insert (contact_states,
+				     g_object_ref (contact),
 				     g_object_ref (presence));
 	}
 }
@@ -596,7 +605,7 @@ notify_event_added_cb (GossipEventManager *event_manager,
 		       gpointer            user_data)
 {
 	GossipEventType type;
-	
+
 	type = gossip_event_get_type (event);
 
 	if (type == GOSSIP_EVENT_NEW_MESSAGE) {
@@ -612,14 +621,14 @@ notify_event_added_cb (GossipEventManager *event_manager,
 		 */
 		if (!g_hash_table_lookup (message_notifications, contact)) {
 			notify = notify_new_message (event_manager, message);
-			
+
 			if (notify) {
 				g_hash_table_insert (message_notifications,
 						     contact,
 						     g_object_ref (event));
-				g_hash_table_insert (event_notifications,   
+				g_hash_table_insert (event_notifications,
 						     g_object_ref (notify),
-						     g_object_ref (event));  
+						     g_object_ref (event));
 			}
 		}
 	}
@@ -639,7 +648,7 @@ notify_message_remove_foreach (gpointer     key,
 	if (gossip_event_equal (event, event_to_compare)) {
 		return TRUE;
 	}
-	
+
 	return FALSE;
 }
 
@@ -652,7 +661,7 @@ notify_event_remove_foreach (gpointer     key,
 		notify_notification_close (key, NULL);
 		return TRUE;
 	}
-	
+
 	return FALSE;
 }
 
@@ -661,10 +670,10 @@ notify_event_removed_cb (GossipEventManager *event_manager,
 			 GossipEvent        *event,
 			 gpointer            user_data)
 {
-	g_hash_table_foreach_remove (message_notifications, 
+	g_hash_table_foreach_remove (message_notifications,
 				     (GHRFunc) notify_message_remove_foreach,
 				     event);
-	g_hash_table_foreach_remove (event_notifications, 
+	g_hash_table_foreach_remove (event_notifications,
 				     (GHRFunc) notify_event_remove_foreach,
 				     event);
 }
@@ -676,7 +685,7 @@ gossip_notify_set_attach_widget (GtkWidget *new_attach_widget)
 		g_object_remove_weak_pointer (G_OBJECT (attach_widget),
 					      (gpointer) &attach_widget);
 	}
-	
+
 	attach_widget = new_attach_widget;
 	if (attach_widget) {
 		g_object_add_weak_pointer (G_OBJECT (new_attach_widget),
@@ -684,6 +693,21 @@ gossip_notify_set_attach_widget (GtkWidget *new_attach_widget)
 	}
 }
 
+void
+gossip_notify_set_attach_status_icon (GtkStatusIcon *new_attach)
+{
+	if (attach_status_icon) {
+		g_object_remove_weak_pointer (G_OBJECT (attach_status_icon),
+					      (gpointer) &attach_status_icon);
+	}
+
+	attach_status_icon = new_attach;
+	if (attach_status_icon) {
+		g_object_add_weak_pointer (G_OBJECT (attach_status_icon),
+					   (gpointer) &attach_status_icon);
+	}
+}
+
 static void
 notify_hint_closed_cb (NotifyNotification *notification,
 		       gpointer            user_data)
@@ -704,7 +728,7 @@ notify_hint_closed_cb (NotifyNotification *notification,
 }
 
 gboolean
-gossip_notify_hint_show (const gchar        *conf_path, 
+gossip_notify_hint_show (const gchar        *conf_path,
 			 const gchar        *summary,
 			 const gchar        *message,
 			 GFunc               func,
@@ -718,15 +742,15 @@ gossip_notify_hint_show (const gchar        *conf_path,
 	g_return_val_if_fail (conf_path != NULL, FALSE);
 	g_return_val_if_fail (summary != NULL, FALSE);
 
-	ok = gossip_conf_get_bool (gossip_conf_get (), 
-				   conf_path, 
+	ok = gossip_conf_get_bool (gossip_conf_get (),
+				   conf_path,
 				   &show_hint);
-	
+
 	if (ok && !show_hint) {
 		return FALSE;
 	}
 
-	notify = notify_notification_new (summary, message, NULL, attach_widget);
+	notify = notify_notification_new (summary, message, NULL, NULL);
 	g_object_set_data_full (G_OBJECT (notify), "conf_path", g_strdup (conf_path), g_free);
 	g_object_set_data (G_OBJECT (notify), "func", func);
 	g_signal_connect (notify,
@@ -734,6 +758,10 @@ gossip_notify_hint_show (const gchar        *conf_path,
 			  G_CALLBACK (notify_hint_closed_cb),
 			  user_data);
 
+	if (attach_status_icon) {
+		notify_notification_attach_to_status_icon (notify, attach_status_icon);
+	}
+
 	if (!notify_notification_show (notify, &error)) {
 		g_warning ("Failed to send notification: %s",
 			   error->message);
@@ -749,13 +777,13 @@ gossip_notify_init (GossipSession      *session,
 {
 	g_return_if_fail (GOSSIP_IS_SESSION (session));
 	g_return_if_fail (GOSSIP_IS_EVENT_MANAGER (event_manager));
-	
+
 	gossip_debug (DEBUG_DOMAIN, "Initiating...");
 
 	if (inited) {
 		return;
 	}
-	
+
 	if (!notify_init (PACKAGE_NAME)) {
 		g_warning ("Cannot initialize Notify integration");
 		return;
@@ -798,7 +826,7 @@ gossip_notify_init (GossipSession      *session,
 			  G_CALLBACK (notify_event_removed_cb),
 			  NULL);
 
- 	inited = TRUE;
+	inited = TRUE;
 }
 
 void
@@ -812,6 +840,6 @@ gossip_notify_finalize (void)
 	g_hash_table_destroy (contact_states);
 	g_hash_table_destroy (message_notifications);
 	g_hash_table_destroy (event_notifications);
-	
+
 	notify_uninit ();
 }
